import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * This class creates a window for the game and performs the drawing of the
 * game objects. It also detects and returns clicks on the game window.
 */
@SuppressWarnings("serial")
public class GameFrame extends JFrame {
    
    private final Walls walls;
    private final GamePanel panel;
    private final List<Ball> balls = new ArrayList<Ball>();
    private Point click;
    private String message;
    
    /**
     * Constructs a game window using the dimensions of the given Walls object.
     * Note that the size of the window is fixed but this class will translate
     * all screen coordinate to and from the coordinate system as defined by the
     * width and height of the Walls object.
     * @param walls bounding walls of the game
     */
    public GameFrame(Walls walls) {
        this.walls = walls;
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setTitle("CS 100m 2007sp Project 5");
        panel = new GamePanel();
        getContentPane().add(panel);
        pack();
        setResizable(false);
        setVisible(true);
    }
    
    /**
     * Adds the given ball to the frame show it will be drawn on the screen when
     * refresh is called.
     * @param ball the ball to add
     */
    public void addBall(Ball ball) {
        synchronized (balls) {
            balls.add(ball);
        }
    }
    
    /**
     * If the user has made a click, returns where the click occurred. If no
     * clicks have happened since the last call to getClick, null is returned.
     * @return the point clicked, or null if no clicks occurred
     */
    public Point getClick() {
        synchronized (panel) {
            Point p = click;
            click = null;
            return p;
        }
    }
    
    /**
     * Sets a message to display on the screen.
     * @param message the message to display
     */
    public void setMessage(String message) {
        synchronized (panel) {
            this.message = message;
        }
    }
    
    /**
     * Redraws the frame after updates have occurred.
     */
    public void refresh() {
        repaint();
        
        try {
            Thread.sleep(5);
        } catch (InterruptedException e) {}
    }

    private static final Color[] colors = new Color[] {Color.BLUE, Color.CYAN, 
            Color.GREEN, Color.MAGENTA, Color.ORANGE, Color.PINK, Color.RED, 
            Color.YELLOW};
    private static final Composite opaque = 
        AlphaComposite.getInstance(AlphaComposite.SRC_OVER);
    private static final Composite faded = 
        AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f);
    private static final Font messageFont = 
        new Font("SansSerif", Font.BOLD, 32);
    
    private class GamePanel extends JPanel implements MouseListener {
        
        public GamePanel() {
            setOpaque(true);
            setDoubleBuffered(true);
            setBackground(Color.DARK_GRAY);
            addMouseListener(this);
        }
        
        public Dimension getPreferredSize() {
            double ratio = walls.getHeight() / walls.getWidth();
            return new Dimension(600, (int) Math.round(600 * ratio));
        }
        
        public Dimension getMinimumSize() {
            return getPreferredSize();
        }
        
        public Dimension getMaximumSize() {
            return getPreferredSize();
        }
        
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            Graphics2D g2d = (Graphics2D) g;
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
            
            synchronized (balls) {
                for (Ball b : balls) {
                    int d = toInt(2 * b.getRadius());
                    
                    if (d <= 0) {
                        continue;
                    }
                    
                    int x = toInt(b.getCenter().getX() - b.getRadius());
                    int y = toInt(b.getCenter().getY() - b.getRadius());
                    Color c = colors[b.getColor() % colors.length];
                    
                    if (b.isExploding()) {
                        g2d.setComposite(faded);
                    } else {
                        g2d.setComposite(opaque);
                    }
                    
                    g.setColor(c);
                    g.fillOval(x, y, d, d);
                }
            }
            
            synchronized (this) {
                if (message != null) {
                    g2d.setColor(Color.WHITE);
                    g2d.setComposite(faded);
                    g2d.setFont(messageFont);
                    g2d.drawString(message, 50, 50);
                }
            }
        }
        
        private int toInt(double x) {
            return (int) Math.round(x / walls.getWidth() * getWidth());
        }
        
        private double toDouble(int x) {
            return (double) x / getWidth() * walls.getWidth();
        }

        public void mousePressed(MouseEvent e) {
            synchronized (this) {
                if (click == null) {
                    click = new Point(toDouble(e.getX()), toDouble(e.getY()));
                }
            }
        }
        
        public void mouseClicked(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mouseReleased(MouseEvent e) {}
        
    }

}
